@@ -20,7 +20,7 @@ require Exporter;
 @EXPORT_OK = qw( NONE ERROR STATUS PROCESS SOCKET HEADERS FILTERS CONNECT ALL );
 %EXPORT_TAGS = ( log => [@EXPORT_OK] );    # only one tag
 
-$VERSION = '0.16';
+$VERSION = '0.18';
 
 my $CRLF = "\015\012";                     # "\r\n" is not portable
 
@@ -38,15 +38,30 @@ use constant HEADERS => 16;    # HTTP headers
 use constant FILTERS => 32;    # Messages from filters
 use constant DATA    => 64;    # Data received by the filters
 use constant CONNECT => 128;   # Data transmitted by the CONNECT method
-use constant ALL     => 255;   # All of the above
+use constant ENGINE  => 256;   # Internal information from the Engine
+use constant ALL     => 511;   # All of the above
 
 # Methods we can forward
-@METHODS = (
-    qw( OPTIONS GET HEAD POST PUT DELETE TRACE CONNECT ), # HTTP
-    qw( COPY MOVE PROPFIND PROPPATCH LOCK UNLOCK MKCOL ), # WebDAV
-    qw( VERSION-CONTROL MKWORKSPACE CHECKOUT CHECKIN
-        MKACTIVITY UPDATE REPORT ),                       # DeltaV
-);
+my %METHODS;
+
+# HTTP (RFC 2616)
+$METHODS{http} = [qw( CONNECT DELETE GET HEAD OPTIONS POST PUT TRACE )];
+
+# WebDAV (RFC 2518)
+$METHODS{webdav} = [
+    @{ $METHODS{http} },
+    qw( COPY LOCK MKCOL MOVE PROPFIND PROPPATCH UNLOCK )
+];
+
+# Delta-V (RFC 3253)
+$METHODS{deltav} = [
+    @{ $METHODS{webdav} },
+    qw( BASELINE-CONTROL CHECKIN CHECKOUT LABEL MERGE MKACTIVITY
+        MKWORKSPACE REPORT UNCHECKOUT UPDATE VERSION-CONTROL ),
+];
+
+# the whole method list
+@METHODS = HTTP::Proxy->known_methods();
 
 # useful regexes (from RFC 2616 BNF grammar)
 my %RX;
@@ -105,6 +120,16 @@ sub new {
     return $self;
 }
 
+sub known_methods {
+    my ( $class, @args ) = @_;
+
+    @args = map { lc } @args ? @args : ( keys %METHODS );
+    exists $METHODS{$_} || carp "Method group $_ doesn't exist"
+        for @args;
+    my %seen;
+    return grep { !$seen{$_}++ } map { @{ $METHODS{$_} || [] } } @args;
+}
+
 sub timeout {
     my $self = shift;
     my $old  = $self->{timeout};
@@ -178,7 +203,7 @@ sub start {
     my $self = shift;
 
     $self->init;
-    $SIG{INT} = $SIG{KILL} = sub { $self->{loop} = 0 };
+    $SIG{INT} = $SIG{TERM} = sub { $self->{loop} = 0 };
 
     # the main loop
     my $engine = $self->engine;
@@ -270,7 +295,13 @@ sub serve_connections {
                       . ":" . $conn->peerport );
 
     my ( $last, $served ) = ( 0, 0 );
-    while ( my $req = $conn->get_request() ) {
+
+    while ( $self->loop() ) {
+        my $req;
+        {
+            local $SIG{INT} = local $SIG{TERM} = 'DEFAULT';
+            $req = $conn->get_request();
+        }
 
         $served++;
 
@@ -1116,6 +1147,16 @@ for any interface.
 This default prevents your proxy to be used as an anonymous proxy
 by script kiddies.
 
+=item known_methods( @groups ) (read-only)
+
+This method returns all HTTP (and extensions to HTTP) known to
+C<HTTP::Proxy>. Methods are grouped by type. Known method groups are:
+C<HTTP>, C<WebDAV> and C<DeltaV>.
+
+Called with an empty list, this method will return all known methods.
+This method is case-insensitive, and will C<carp()> if an unknown
+group name is passed.
+
 =item logfh
 
 A filehandle to a logfile (default: *STDERR).
@@ -1137,6 +1178,7 @@ are powers of 2, starting from 0 and listed here in ascending order):
     FILTERS - Filter information
     DATA    - Data received by the filters
     CONNECT - Data transmitted by the CONNECT method
+    ENGINE  - Engine information
     ALL     - Log all of the above
 
 If you only want status and process information, you can use:
@@ -1328,7 +1370,7 @@ Maximum number of concurrent TCP connections (i.e. child processes).
 
 =item max_requests_per_child
 
-Maximum number of TCP connections handled by the smae child process.
+Maximum number of TCP connections handled by the same child process.
 
 =item min_spare_servers
 
